#!/bin/bash
# Adapted from the "log" at...
# http://uml.nagafix.co.uk/Debian-3.1/install.html

# Configuration:
source config.sh
export IMAGEDIR=`pwd`"/image"
export ISOMOUNTDIR=`pwd`"/iso"
export MIRROR="file://"$ISOMOUNTDIR"/"

echo "Getting the ISO file..."
if [ ! -e $ISOFILE ]; then
	wget -O $ISOFILE $ISOADDRESS
fi

echo "Making the filesystem with size ${IMAGESIZE}GB..."
dd if=/dev/zero of=$IMAGE bs=1024 count=1 seek=$((${IMAGESIZE}*1024*1024-1024))
mkfs.ext3 $IMAGE
chown $IMAGEUSER.$IMAGEGROUP $IMAGE

echo "Making the swap file with size ${SWAPSIZE}MB..."
dd if=/dev/zero of=$SWAPIMAGE bs=1M count=$SWAPSIZE
chown $IMAGEUSER.$IMAGEGROUP $SWAPIMAGE

echo "Accessing the ISO file..."
if [ ! -e $ISOMOUNTDIR ]; then
	mkdir $ISOMOUNTDIR
fi
mount -o loop $ISOFILE $ISOMOUNTDIR

echo "Accessing the image..."
if [ ! -e $IMAGEDIR ]; then
	mkdir $IMAGEDIR
fi
mount -o loop $IMAGE $IMAGEDIR

echo "Attempting to debootstrap the image from the ISO..."
# Hoary has debootstrap 0.2.45ubuntu27 but needs Dapper's debootstrap 0.3.3.0ubuntu2
# for the right files.
# Using "debootstrap", not "linux32 debootstrap" which is supposedly needed for amd64.
debootstrap --arch i386 $DISTNAME $IMAGEDIR $MIRROR

echo "Creating special UML devices..."
# create the ubd devices (missing from Debian log)
if [ ! -e "$IMAGEDIR/dev/ubda" ]; then
	if [ -e "$IMAGEDIR/dev" ]; then
		for ((i=0; i<8; i++))
		do
			let=$(echo $i|tr '0-7' 'a-z')
			mknod $IMAGEDIR/dev/ubd$let b 98 $[16 * $i]
			for ((p=1; p<=16; p++)) do
				mknod $IMAGEDIR/dev/ubd$let$p b 98 $[16 * $i + $p]
			done
		done
	else
		echo "/dev not found!"
		exit 1
	fi
fi

echo "Copying host DNS configuration..."
cp /etc/resolv.conf $IMAGEDIR/etc/

echo "Copying filesystems configuration..."
cp fstab $IMAGEDIR/etc/

echo "Making a mount directory for the ISO..."
mkdir $IMAGEDIR/mnt/cdrom

echo "Creating network configuration..."
echo "$GUESTADDRESS $GUESTNAME" > hosts
echo "127.0.0.1 localhost" >> hosts
echo "$HOSTADDRESS $HOSTNAME" >> hosts
chown $IMAGEUSER.$IMAGEGROUP hosts

echo "auto lo" > interfaces
echo "iface lo inet loopback" >> interfaces
echo "iface eth0 inet static" >> interfaces
echo "    network 192.168.0.0" >> interfaces
echo "    broadcast 192.168.0.255" >> interfaces
echo "    address $GUESTADDRESS" >> interfaces
echo "    netmask 255.255.255.0" >> interfaces
echo "    up route add default gw $HOSTADDRESS" >> interfaces
echo "" >> interfaces
echo "auto eth0" >> interfaces
chown $IMAGEUSER.$IMAGEGROUP interfaces

echo "Copying network configuration..."
cp hosts $IMAGEDIR/etc/
cp interfaces $IMAGEDIR/etc/network/

echo "Copying filesystems configuration..."
cp $DISTNAME/sources.list $IMAGEDIR/etc/apt/

#echo "Copying kernel modules configuration..."
#cp sound/modules $IMAGEDIR/etc/modules
#cp sound/modules-hostaudio $IMAGEDIR/etc/modprobe.d/hostaudio

echo "Reconfiguring the console settings..."
# tweak the inittab to only use tty0 and add it to securetty
cp $IMAGEDIR/etc/inittab $IMAGEDIR/etc/inittab.save
grep -v "getty" $IMAGEDIR/etc/inittab.save > $IMAGEDIR/etc/inittab
echo "# We launch just one console for UML:" >> $IMAGEDIR/etc/inittab
echo "c0:1235:respawn:/sbin/getty 38400 tty0 linux" >> $IMAGEDIR/etc/inittab

echo "# UML modification: use tty0 or vc/0" >> $IMAGEDIR/etc/securetty
echo "tty0" >> $IMAGEDIR/etc/securetty
echo "vc/0" >> $IMAGEDIR/etc/securetty

echo "Copying useful scripts..."
cp swap-guest.sh xephyr-guest.sh user-guest.sh install-guest.sh $IMAGEDIR/usr/local/bin
cp config.sh $IMAGEDIR/usr/local/bin/guest-config.sh

echo "Unmounting the image..."
umount $IMAGEDIR
rmdir $IMAGEDIR

echo "Unmounting the ISO..."
umount $ISOMOUNTDIR
rmdir $ISOMOUNTDIR

echo "--------"
echo "To start networking:"
echo "[host] ./start-net.sh"
echo
echo "To start the virtual machine:"
echo "[host] ./start.sh"
echo
echo "After starting the virtual machine, log in as root (no password)."
echo
echo "If swap is not enabled, run the following command:"
echo "[guest] swap-guest.sh"
echo
echo "To add a new user:"
echo "[guest] user-guest.sh $IMAGEUSER"
echo "(Or use another username instead of ${IMAGEUSER}.)"
echo
echo "To install Xephyr (Xnest) and KDE:"
echo "[guest] install-guest.sh"
echo
echo "It might be quickest to reboot the virtual machine at this point:"
echo "[guest] reboot"
echo
echo "--------"
echo "Networking, if not activated, can be started as follows:"
echo "[guest] /etc/init.d/networking start"
echo
echo "To start KDE manually:"
echo "[guest] /etc/init.d/kdm start"
echo
echo "To stop the virtual machine:"
echo "[guest] halt"
echo
echo "--------"
echo "To stop networking:"
echo "[host] ./stop-net.sh"
