#!/bin/bash

# Configuration:
source config.sh
export IMAGEDIR=`pwd`"/image"

echo "Getting the Linux kernel archive..."
if [ ! -e "$LINUXARCHIVE" ]; then
	wget -O $LINUXARCHIVE $LINUXADDRESS
fi

echo "Unpacking the archive..."
if [ ! -e "`pwd`/$LINUXDIRNAME" ]; then
	tar jxf $LINUXARCHIVE
fi

# Copy our configuration into the unpacked directory.
# Normally we would do this:
# make defconfig ARCH=um
# make menuconfig ARCH=um
# In "File systems", "CD-ROM/DVD Filesystems", press Y for "ISO 9660...".
# The supplied configuration file should include this information.
cp ${LINUXDIRNAME}/.config ${LINUXDIRNAME}/.config.original
cp linux-build.config ${LINUXDIRNAME}/.config

# Enter the unpacked directory and build.
cd $LINUXDIRNAME

#echo "Patching hostaudio for SDL..."
#patch -p0 < ../patches/hostaudio_kern.c.diff

echo "Building..."
make ARCH=um

# Mount the root filesystem and install the modules.
echo "Making the modules..."
make modules ARCH=um

echo "Accessing the image..."
if [ ! -e $IMAGEDIR ]; then
        mkdir $IMAGEDIR
fi
mount -o loop $IMAGE $IMAGEDIR

echo "Installing the modules..."
make modules_install INSTALL_MOD_PATH=$IMAGEDIR ARCH=um

echo "Unmounting the image..."
umount $IMAGEDIR
rmdir $IMAGEDIR

# Now get the executable and remove the build directory.
cd ..
mv ${LINUXDIRNAME}/linux .

if [ -e "`pwd`/linux" ]; then
	echo "To remove the Linux kernel sources, do this:"
	echo "sudo rm -rf ${LINUXDIRNAME}"
fi
